//-----------------------------------------
// LScript Modeler template
//
// Copyright 2018 Michael Green
// mike@mikegreen.name
//

myversion=2;
//first finished version
// V2 Re written to automate the growing of multiple hair groups.

@version 2.2
@warnings
@script modeler

// global values go here
st_secti=1;
st_hairs=2;
st_roots=3;
st_inset=4;
st_minim=5;
st_subdl=6;
st_parts=7;

gset; 		//The settings.

InitSet
{
	gset[st_secti]=10;          //Number of sections going to be made.
	gset[st_hairs]="Hair";		//Hair surface name
	gset[st_roots]="Root";      //Root Surface
	gset[st_inset]=.002;         // How far the root is inset into the head.
	gset[st_minim]=.005;		// Minimum distance the roots are apart.
	gset[st_subdl]=1;			// Number of times to subdivide the surface.
	gset[st_parts]=1;			// Use parts instead of surfaces.
}


main
{
	InitSet();
	for(i=1;i<=gset.size();i++)
	{
		gset[i]=recall("MG_WizzyWig2"+i.asStr(),gset[i]);
	}
    // requester code goes here

    reqbegin("WizzyWig2 by Mike Green. Version"+myversion);

    c[st_secti] = ctlinteger("No. of segments to each hair",			gset[st_secti]);
    c[st_hairs] = ctlsurface("Hair Surface",							gset[st_hairs]);
    c[st_roots] = ctlsurface("Root Surface",							gset[st_roots]);
    c[st_inset] = ctldistance("Root inset distance",					gset[st_inset]);
    c[st_minim] = ctldistance("Minimum distance of roots",				gset[st_minim]);
    c[st_subdl] = ctlinteger("Subdivision Level",						gset[st_subdl]);
    c[st_parts] = ctlcheckbox("Use Part Names",							gset[st_parts]);

	h1 = ctlpopup("Help",1,@
					"Email mike@mikegreen.name",
					"1>Place some polygons you want the hair to","grow from in the background layer.",
					"2>Place some guide curves you want the hair","to grow along in the foreground layer.",
					"3>Press ok and the wig will be grown",
					" ",
					"Guide splines will only grow hair from polygons with the same SURFACE as themselves.",
					"This way you can use groups of splines with the same surface to grow clumps of hair",
					"with a parting",
					"Root Inset > How deep into the head you want the roots."@);

    return if !reqpost();

    // get requester control values here
    for(i=1;i<=gset.size();i++)
    {
		gset[i]=getvalue(c[i]);
		store("MG_WizzyWig2"+i.asStr(),gset[i]);
	}

    reqend();
    mysurfaces=CountSurfaces();
    if(!mysurfaces)
		error("No curves found in FG layer");

	mybg=lyrbg();
	if(!mybg) error("No BG layer selected");

	undogroupbegin();

	myfg=lyrfg();
	mybg=lyrbg();
  	temp=lyrempty();
  	mywiglyr=temp[1];
	mylyrlist;
	for(i=1;i<=mysurfaces.size();i++)
	{
		// do a surface at a time
  		mylyrlist[i]=MakeHairGroup(gset,myfg,mybg,mywiglyr,mysurfaces[i]);
	}
	mylyrlist.pack();
	if(mylyrlist[1]==nil)
		error("No scalp polygons found");
	lyrsetfg(mylyrlist);
	cut();
	lyrsetfg(mywiglyr);
	paste();
	selmode(USER);
	selpolygon(SET, NVEQ, 1);
	cmdseq("Sel Points");
	mycount=pointcount();
	editbegin();
	mymap=VMap(VMWEIGHT,"Hairweight",1);
	editend();
	mypoints;
	mypointcount=mycount*(gset[st_secti]+1);
	moninit(mypointcount,"Processing "+mypointcount.asStr()+ " points");
	for(i=1;i<=gset[st_secti]+1;i++)
	{
		editbegin();
		foreach(p,points)
		{
			mymap.setValue(p,(i - 1)/gset[st_secti]);
			mypoints+=p;
			monstep();
		}
		editend();
		cmdseq("Select Expand");
		selpoint(CLEAR, POINTID, mypoints);

	}
	monend();
  	undogroupend();
}


CountSurfaces
{	
	mycurvesurfaces=nil;
	mycount=0;
	selmode(DIRECT);
	selpolygon(CLEAR);
	selpolygon(SET,CURVE);
	selhide(UNSELECTED);
	undogroupbegin();
	if(gset[st_parts])
		cmdseq("Parts to Surfs");
	mysurface=Surface();
	while(mysurface)
	{
		selpolygon(CLEAR);
		selpolygon(SET,SURFACE,mysurface.name);
		mypolys=polycount();
		if(mypolys[1]!=0)
		{
			mycount++;
			mycurvesurfaces[mycount]=mysurface.name;
  		}
  		mysurface=mysurface.next();
	}
	undogroupend();
	if(gset[st_parts])
		undo();
	selpolygon(CLEAR);
	return(mycurvesurfaces);
}

MakeHairGroup:pset,pfg,pbg,pwiglyr,psurface
{
   	selmode(USER);
   	if(gset[st_parts])
		selpolygon(SET, PART, psurface); // Select Part of guides
	else
		selpolygon(SET, SURFACE, psurface); // Select surface of guides
	copy(); // selhide() doesn't work, so lets workaround.
	lyrsetfg(pwiglyr);
	paste();
   	lyrsetfg(pbg); // Set scalp layer

   	if(gset[st_parts])
		selpolygon(SET, PART, psurface); // Select Part of head
	else
		selpolygon(SET, SURFACE, psurface); // Select surface of head
   	selmode(DIRECT);
	mycount=polycount();
	if (mycount[1]==0)
	{
		lyrsetfg(pwiglyr);
		selmode(USER);
		delete();
  		lyrsetfg(pfg);  // Go to guide layer
  		lyrsetbg(pbg);   // Bg scalp
		return(nil);
	}
	copy();
	templyr=lyrempty();
	lyrsetfg(templyr[1]);
    paste();
	MakeRoots(pset);
    copy();

//     if(pset[st_slite])
// 		pset[st_secti]=CalcSasLimit(pset[st_group]);
	lyrsetbg(pwiglyr);
  	railextrude(pset[st_secti],LENGTHS,"s");
  	changesurface(pset[st_hairs]);
  	paste();          // paste roots back in.
  	mergepoints();    // merge the roots to the hairs.
  	lyrsetfg(pwiglyr);  // Go to guide layer
  	delete();
  	lyrsetfg(pfg);  // Go to guide layer
  	lyrsetbg(pbg);   // Bg scalp
  	return(templyr[1]);
}

HidePrevious
{
    tempcount=polycount();
    if (tempcount[1]+pointcount()>0)
    {
        selmode(GLOBAL);
        selhide(SELECTED);
        selmode(USER);
    }
}

MakeRoots:pset
{
    selmode(USER);
		smoothscale(0 - pset[st_inset]);
    temp=0;
    while (temp<pset[st_subdl])
    {
        subdivide(METAFORM);
        temp++;
    }
    removepols();
    mergepoints(pset[st_minim]);
    editbegin();
        mycounter=1;
        while (mycounter <= points.size())
        {
            addpolygon(points[mycounter],pset[st_roots]);
            mycounter++;
        }
    editend();
}




